"""
Arithmetic tasks for training reasoning capabilities:
"""

from .list_functions import ListFunctionsDataset, ListFunctionsDatasetConfig

__all__ = [
    "ListFunctionsDataset",
    "ListFunctionsDatasetConfig",
]
